@@ -7,19 +7,19 @@
 )]
 
 use {
-    git2::{Commit, Oid},
-    std::{cell::RefCell, cmp::max, collections::HashMap, path::PathBuf, process::Command, rc::Rc},
-    thousands::Separable,
-    tracing::{debug_span, instrument},
-    ::{
-        clap::Parser,
-        digest::Digest,
-        eyre::{bail, Result, WrapErr},
-        git2::{ErrorCode, Repository, RepositoryInitOptions, Signature, Time},
-        rayon::prelude::*,
-        std::{env, fs},
-        tracing::{debug, info, trace, warn},
+    clap::Parser,
+    digest::Digest,
+    eyre::{bail, Result, WrapErr},
+    git2::{
+        Commit, ErrorCode, Oid, Repository, RepositoryInitOptions, RepositoryState, Signature, Time,
     },
+    rayon::prelude::*,
+    std::{
+        cell::RefCell, cmp::max, collections::HashMap, env, fs, path::PathBuf, process::Command,
+        rc::Rc,
+    },
+    thousands::Separable,
+    tracing::{debug, debug_span, info, instrument, trace, warn},
 };
 
 /// Would you like to SAVE the change?
@@ -45,7 +45,7 @@ pub struct Args {
     #[clap(long = "hash", short = 'x')]
     pub hash_hex: Option<String>,
 
-    /// Use a manual commit message instead of the default generated message.
+    /// Specify a commit message to use instead of the default.
     #[clap(long, short = 'm')]
     pub message: Option<String>,
 
@@ -65,8 +65,28 @@ pub struct Args {
     #[clap(long, short = 'q', parse(from_occurrences))]
     pub quiet: i32,
 
+    /// Squash/amend previous commit(s), instead of adding a new one.
+    ///
+    /// By default, `--squash` will behave like `git commit --amend`, only
+    /// replacing the most recent commit. However, specifying a larger number
+    /// such as `--squash=2` will squash that many recent commits (and any
+    /// current changes) into a single commit. If any of those commits are
+    /// merges, any non-squashed parents will be added as parents of the
+    /// squashed commit.
+    #[clap(
+        long = "squash",
+        alias = "amend",
+        short = 's',
+        default_value = "0",
+        default_missing_value = "1"
+    )]
+    pub squash_commits: u32,
+
     /// Seconds of timestamp allocated for each commit to search.
-    #[clap(long="step", short='s', default_value_t = 64 * 2)]
+    ///
+    /// The number of possibilities searched is the half the square of this
+    /// value.
+    #[clap(long="step", short='t', default_value_t = 64 * 2)]
     pub step_seconds: u32,
 
     /// Increase log verbosity. May be used multiple times.
@@ -129,6 +149,17 @@ pub fn main(args: Args) -> Result<()> {
         },
     };
 
+    if args.squash_commits > 0 {
+        todo!("--squash has not been implemented");
+    }
+
+    if repo.state() != RepositoryState::Clean {
+        bail!(
+            "Repository is in the middle of another operation: {:?}",
+            repo.state()
+        );
+    }
+
     let head = match repo.head() {
         Ok(head) => Some(head.peel_to_commit().unwrap()),
         Err(err) if err.code() == ErrorCode::UnbornBranch => None,
@@ -201,9 +232,9 @@ pub fn main(args: Args) -> Result<()> {
         placeholder_email.to_string()
     };
 
-    let generation_index = head
+    let generation_number = head
         .as_ref()
-        .map(|commit| generation_index(commit) + 1)
+        .map(|commit| generation_number(commit) + 1)
         .unwrap_or(0);
 
     let mut index = repo.index()?;
@@ -254,7 +285,7 @@ pub fn main(args: Args) -> Result<()> {
     let tree4 = &tree.to_string()[..4];
     let tree = repo.find_tree(tree)?;
 
-    let revision_index = generation_index + 1;
+    let revision_index = generation_number + 1;
     let message = args.message.unwrap_or_else(|| {
         let mut message = format!("r{}", revision_index);
         if let Some(ref head) = head {
@@ -307,7 +338,7 @@ pub fn main(args: Args) -> Result<()> {
         .wrap_err("commit must be valid utf-8")
         .unwrap();
 
-    let (author_timestamp, commit_timestamp, brute_hash, commit_buffer) =
+    let (author_timestamp, commit_timestamp) =
         brute_force_timestamps(base_commit, &target_hash, min_timestamp, max_timestamp);
 
     if !args.dry_run {
@@ -329,16 +360,6 @@ pub fn main(args: Args) -> Result<()> {
             &tree,
             parents,
         )?;
-    } else {
-        info!(
-            "Skipping commit write because this is a dry run:\n\ncommit {}\n{}",
-            brute_hash
-                .iter()
-                .map(|b| format!("{:02x}", b))
-                .collect::<Vec<_>>()
-                .join(""),
-            commit_buffer
-        );
     }
 
     eprintln!();
@@ -361,7 +382,9 @@ pub fn main(args: Args) -> Result<()> {
     Ok(())
 }
 
-/// Given a raw Git commit as a string, find the timestamps in the given range
+/// Brute forces timestamps for a Git commit.
+///
+/// Given a raw Git commit as a string, finds the timestamps in the given range
 /// that will produce the closest commit ID to target_hash. We ensure that
 /// min_timestamp <= author_timestamp <= committer_timestamp <= max_timestamp
 /// because it would be weird to it committed before being authored.
@@ -371,7 +394,7 @@ pub fn brute_force_timestamps(
     target_hash: &[u8],
     min_timestamp: i64,
     max_timestamp: i64,
-) -> (i64, i64, Vec<u8>, String) {
+) -> (i64, i64) {
     let base_commit_lines = base_commit.split('\n').collect::<Vec<&str>>();
     let author_line_index = base_commit_lines
         .iter()
@@ -408,7 +431,7 @@ pub fn brute_force_timestamps(
         commit_lines.join("\n")
     };
 
-    let (_score, author_timestamp, commit_timestamp, hash, candidate) = ((min_timestamp
+    let (_score, author_timestamp, commit_timestamp, hash, _candidate) = ((min_timestamp
         ..=max_timestamp)
         .into_par_iter()
         .map(|author_timestamp| {
@@ -437,16 +460,25 @@ pub fn brute_force_timestamps(
     .min()
     .unwrap();
 
-    (author_timestamp, commit_timestamp, hash, candidate)
+    debug!(
+        "Brute-forced a commit with id: {}",
+        hash.iter()
+            .map(|b| format!("{:02x}", b))
+            .collect::<Vec<_>>()
+            .join("")
+    );
+
+    (author_timestamp, commit_timestamp)
 }
 
-/// Finds the generation index of a given Git commit.
+/// Finds the generation number of a given Git commit.
 ///
 /// The generation index is the number of edges of the longest path between the
 /// given commit and an initial commit (one with no parents, which has an
-/// implicit generation index of 0).
+/// implicit generation index of 0). The Git documentation also refers to this
+/// as the "topological level" of a commit (https://git-scm.com/docs/commit-graph).
 #[instrument(level = "debug")]
-pub fn generation_index(commit: &Commit) -> u32 {
+pub fn generation_number(commit: &Commit) -> u32 {
     let head = commit.clone();
 
     #[derive(Debug, Clone)]
@@ -524,11 +556,11 @@ pub fn generation_index(commit: &Commit) -> u32 {
         (head, initial_commits)
     };
 
-    let generation_index = {
+    let generation_number = {
         let span = debug_span!("measure_git_graph");
         let _guard = span.enter();
 
-        let mut generation_index = 0;
+        let mut generation_number = 0;
 
         let mut live = vec![root];
 
@@ -536,7 +568,7 @@ pub fn generation_index(commit: &Commit) -> u32 {
             let commit = commit.borrow_mut();
 
             if commit.edges_out.is_empty() {
-                generation_index = max(generation_index, commit.max_distance_from_head);
+                generation_number = max(generation_number, commit.max_distance_from_head);
             } else {
                 for parent in commit.edges_out.iter() {
                     let mut parent_mut = parent.borrow_mut();
@@ -553,10 +585,10 @@ pub fn generation_index(commit: &Commit) -> u32 {
             }
         }
 
-        generation_index
+        generation_number
     };
 
-    generation_index
+    generation_number
 }
 
 /// Initialize the typical global environment and parses the typical [Args] for
@@ -601,7 +633,7 @@ pub fn init() -> Args {
             .finish(),
     ));
 
-    trace!("Initialized from {:#?}", args);
+    trace!("Initialized from: {:#?}", args);
 
     args
 }
